<?php // config.php

#   File was released under GNU General Public License 3
#   Articles Copyright (C) 2024  Wloff Apps, Services, Development
#   This program comes with ABSOLUTELY NO WARRANTY;
#=================================================================


# configuration file, settings
# class definitions
# variable definitions
# function definitions


/* ============================== */
/* ==== Class Definitions ==== */
// a class that encapsulates a blog entry, an article, the article of a blog entry; right?
// it only needs a file as an input, and it digests the file and contents
class Article {
    // couple of properties has to be pre-defined, otherwise PHP complains (undefined variable notice)
    public $authorSpan;
    public $etags = [];
    public $etagLinks = [];
    // constructor function to initialize the rest of the properties
    public function __construct($filename, $pathtofile, $catalog = false, $tagview = false, $author = null) {
        // extracting timestamp from the filename
        $this->tstamp = (int)$filename;
        // glue the path of the file together from the path to the directory and the filename
        $this->path = $pathtofile . $filename;
        // here's a check to make sure the file actually exists and the object can be created
        if (!file_exists($this->path)) {
            // if no file found then output an error message
            echo file_get_contents("html/head.html");
            echo file_get_contents("html/header.html");
            exit("<p style='color:var(--article-color)'>Requested blog entry is not found</p>");
        }
        // whatever happens after this point we need the contents of the file and the title of the article
        $this->content = file($this->path);
        // title with the h3 html tags
        $this->title = $this->content[0];
        // title without those
        $this->pureTitle = strip_tags($this->title);
        // from here we have three path
        // either generate the necessary data for the regular catalog
        // or for the catalog tag view
        // or for the full blog post
        if ($catalog === true) {
            // for the regular catalog in addition we only need the year and the month-day
            $this->year = date("Y", $this->tstamp);
            $this->day = date("m. d.", $this->tstamp);
        } elseif ($tagview === true) {
            // for tag view we need the whole year-month-day format
            $this->date = date("Y. F d.", $this->tstamp);
            // and the e-tag list so can compare easily
            foreach($this->content as $value) {
                // sadly have to use regex
                if (preg_match('/<e-tag[^>]*>(.*)<\/e-tag>/', $value)) {
                    // if it matches then put em into an array neatly without the html tags
                    array_push($this->etags, trim(strip_tags($value)));
                }
            }
        } else {
            // when we need to create a whole blogpost, not just links
            // process the front of the file, date, author, title, and intro (ie first paragraph)
            $this->date = date("Y. F d.", $this->tstamp);
            $this->datePara = "<p class='tstamp'>$this->date</p>";
            // we don't always have author, do we
            if (!is_null($author)) {
                $this->auth = $author;
                $this->authorSpan = "<span class='author'>by $this->auth</span>\n";
            }
            // so the title will be a link, not just a heading
            $this->titleLink = "<h3><a class='link' href='blog.php?blog=$this->tstamp'>$this->pureTitle</a></h3>\n";
            // this is for the soc media sharing support
            $this->intro = $this->content[1];
            $this->pureIntro = strip_tags($this->intro);
            // now deal with the other end of the file, the e-tags
            // first get them
            foreach($this->content as $value) {
                // sadly have to use regex
                if (preg_match('/<e-tag[^>]*>(.*)<\/e-tag>/', $value)) {
                    // if it matches then put em into an array neatly without the html tags
                    array_push($this->etags, trim(strip_tags($value)));
                }
            }
            // now convert them to links
            foreach($this->etags as $etag) {
                $etagLink = "<e-tag><a class='link' href='catalog.php?tag=$etag'>$etag</a></e-tag>\n";
                array_push($this->etagLinks, $etagLink);
            }
            // now the text in between (between the title and the e-tags)
            $this->newContent = $this->content;
            array_shift($this->newContent);
            for ($i=0 ; $i < sizeof($this->etagLinks); $i++) {
                array_pop($this->newContent);
            }
        }
    }

// create links for the regular catalog view
// inlcudes the date in "month n day" format, timestamp for the link to the individual article and the text only title (no h3 html tags)
    public function drawCatalogLink() {
        echo "<span>$this->day - <a class='link' href='blog.php?blog=$this->tstamp'>$this->pureTitle</a></span>\n";
    }

// create links for the catalog tag view
// inlcudes the full date with year, timestamp for the link to the individual article, and the text only title
    public function drawCatalogTagLink() {
        return "<span>$this->date - <a class='link' href='blog.php?blog=$this->tstamp'>$this->pureTitle</a></span>\n";
    }

// the main function of the class, constructs a whole article from the building blocks
    public function drawArticle() {
        echo "<article>\n" . $this->datePara . $this->authorSpan . $this->titleLink;
        forEach($this->newContent as $elem) {
            echo $elem;
        }
        echo "<div class='tags'>";
        forEach($this->etagLinks as $elem) {
            echo $elem;
        }
        echo "</div>\n</article>\n";
    }

}


/* == Optional author == */
// Add your name/username/identifier/alias etc. within "" ofc
$author = null;
// example:
#$author = "admin";


/* == Path of directory containing the articles == */
// so don't have to repeat this over and over, and another dir can be set easily
$articlePath = "content/articles/";


/* == Default value for Catalog == */
// this is for the regular catalog view, without tagged category
$tag = null;


/* == Gathering the blog entry files into an array == */
// this is done in three steps
// 1. scanning the directory for catalogs
// 2. filtering out the '.' and '..' directories from the catalogs, so we get only the files
// 3. reversing the order of the files, so when we loop through
// then the newer posts will be displayed first on the home page
$files = array_reverse(filterDots(scandir($articlePath)));


/* == Number of blog entries Per Page - for pagination == */
// how many blog entries are displayed on one page
// change it if you want more
// or less
$entryPerPage = 3;


/* == file number related variable calculations == */
// count the number of files gathered
$entryCount = count($files);
// calculate how many pages have to be generated
$numOfPages = ceil($entryCount / $entryPerPage); 
// rearrange the $files array into a multi level one
// each level contains one page worth of blog entries
$filesPages = array_chunk($files, $entryPerPage);


/* == Number of pages validation == */
// setting a range for the validation filter on the index page
$pagesRange = array(
    "options" => array (
        "min_range" => 1,
        "max_range" => $numOfPages
    )
);


/* ============================== */
/* ==== Function Definitions ==== */
// when gathering the files in a directory into an array
// the method used also gathers the "." and ".." from that directory
// think the bash command ls -a, which also lists them
// "." = current directory
// ".." = one directory above
// this function below gets rid of those, since we can't use them for anything in this context
function filterDots($arr) {
    foreach ($arr as $value) {
        if ($value == '.' || $value == '..') {
            $position = array_search($value, $arr);
            array_splice($arr, $position, 1);
        }
    }
    return $arr;
}


/* ===================================== */
/* ============== Catalog ============== */
// two types of catalog is possible
// one regular and one for a tag
function generateCatalog($arr, $path, $t = null) {
    if (is_null($t)) {
        $n = 1;
        foreach ($arr as $value) {
            // instantiate an article
            $a = new Article($value, $path, true, false);
            // if we are at the first iteration, eg. the first file
            if ($n == 1) {
                // then we note the year of this post, use it in testing
                $oldyear = $a->year;
                // and we open an article to list the blog entries which happened in the same year
                echo "<article><h3>$a->year</h3>\n";
            }
            // then if the year of the first file is larger than the year in the file we process
            // which can happen in the subsequent files (but obvioulsy not in the first)
            // and which indicates that we arrived to another year
            // which means we have to close the previous article of the previos year
            // and open a new article
            if ($oldyear > $a->year) {
                // update the year for tests
                $oldyear = $a->year;
                echo "</article>\n"; // close article of previous year
                echo "<article><h3>$a->year</h3>\n"; // open article for the new year
            }
            // put one link to the page
            $a->drawCatalogLink();
            // update the counter
            $n++;
        }
        // to complete the article of the very last iteration, we finish with closing that article
        echo "</article>\n";
    } else {
        // predefine an array to hold the links if any
        $links = [];
        // loop through the files
        foreach ($arr as $value) {
            // get the text of the blog, contents of the file
            $a = new Article($value, $path, false, true);
            // if the data sent via the GET can be found among the e-tags of an article
            if (in_array($t, $a->etags)) {
                // add it to the array
                array_push($links, $a->drawCatalogTagLink());
            }
        }
        // if anything in the array, any match found
        if ($links) {
            // then open a new article, create a heading for it with that tag
            echo "<article><h3>Tag: $t</h3>\n";
            // fill the article with the links
            foreach($links as $link) {
                echo $link;
            }
            // close the article
            echo "</article>\n";
        } else {
            // if nothing matches, display an error message
            echo "<article><p>The requested tag was not found!</p></article>\n";
        }
    }
}


/* ======================================== */
/* ============== Pagination ============== */
// creating the page numbering to navigate through the blog entries on the home page
function generatePagination($last, $current) {
    // 1. create the container for the page numbers/links (they'll be <a> elements)
    echo "<div id='pagination'>";

    // 2. defining couple of variables for the loop
    // This is needed because we only display numbers with the offset of -3 and +3
    // from the current page we are on, so seven page numbers all in all
    // 
    // 2a. the $i where the counting starts
    if ($current-3 >= 1) {
        $i = $current-3;
    } else {
        $i = 1;
    }
    // 2b. the maximum number the loop runs, this is the max number of the pages
    if ($current+3 > $last) {
        $top = $last;
    } else {
        $top = $current + 3;
    }

    // 3. these two are needed so the first page is always displayed
    if ($current > 4) {
        echo "<a class='link' href='index.php?page=1'>1</a>";
    }
    if ($current > 5) {
        echo "<span>...</span>";
    }

    // 4. the loop which generates the main body of the numbers and prints them out
    for ($i ; $i <= $top ; $i++) {
        if ($i == $current) {
            echo "<a href='index.php?page=$i' class='current link'>$i</a>";
        } else {
            echo "<a class='link' href='index.php?page=$i'>$i</a>";
        }        
    }

    // 5. these two are needed so the last page is always displayed
    if ($current < $last-4) {
        echo "<span>...</span>";
    }
    if ($current < $last-3) {
        echo "<a class='link' href='index.php?page=$last'>$last</a>";
    }

    // 6. close the div
    echo "</div>";
}

?>
